//************************************************************************
//
//	CTXHandler.m
//
//	by Jrgen Mllenhoff <jm@oic.de>		
//
//
//	This code is supplied "as is" the author makes no warranty as to its 
//	suitability for any purpose.  This code is free and may be distributed 
//	in accordance with the terms of the:
//		
//			GNU GENERAL PUBLIC LICENSE
//			Version 2, June 1991
//			copyright (C) 1989, 1991 Free Software Foundation, Inc.
// 			675 Mass Ave, Cambridge, MA 02139, USA
//
//************************************************************************

#import "CTXHandler.h"

#import "HTTPSPreferences.h"
#import "HTTPSFunctions.h"

#import	<OmniFoundation/OFUserDefaults.h>
#import	<OmniBase/rcsid.h>

#import	<Foundation/NSLock.h>

RCS_ID("$Header: /LocalDeveloper/CVS/HTTPS/CTXHandler.m,v 1.7 1998/05/03 19:13:58 jurgen Exp $")

@implementation CTXHandler

static	CTXHandler	*sharedCTXHandler = nil;
static	NSLock		*ctxHandlerLock = nil;

+ (void)initialize 
{
	static BOOL fInitialized = NO;

	[super initialize];
	if (fInitialized)
		return;
    fInitialized = YES;

	ctxHandlerLock = [[NSLock alloc] init];
}

+ (BOOL)isSharedCTXHandler
{
	return (sharedCTXHandler != nil);
}

+ (CTXHandler *)sharedCTXHandler
{
	static BOOL	fInit = NO;
	
	if (sharedCTXHandler != nil)
		return sharedCTXHandler;

	[ctxHandlerLock lock];

	if (!fInit)
	{
		SSLeay_add_ssl_algorithms();
		SSL_load_error_strings();
		fInit = YES;
	}

	if (sharedCTXHandler == nil)
	{
		sharedCTXHandler = [[CTXHandler alloc] init];
		if (sharedCTXHandler == nil)
		{
			NSLog(@"%@:+%@: Failed to init CTXHandler [%@]", 
				NSStringFromClass([self class]), NSStringFromSelector(_cmd), sslErrorString());
		}
	}

	[ctxHandlerLock unlock];

	return sharedCTXHandler;
}

- init
{
	[super init];

	_ctx = SSL_CTX_new(SSLv23_client_method());
	if (_ctx == NULL)
	{
		[self release];
		return nil;
	}

	[self setUserDefaults];

	if (!SSL_CTX_set_default_verify_paths(_ctx)) 
	{
		[self release];
		return nil;
	}

	SSL_CTX_set_session_cache_mode(_ctx, SSL_SESS_CACHE_CLIENT);
	
	return self;
}

- (void)setUserDefaults
{
	if (_ctx == NULL)
		return;
		
	if ([[OFUserDefaults sharedUserDefaults] boolForKey:HTTPSUseBugfixes])
		SSL_CTX_set_options(_ctx, SSL_OP_ALL);
	else
		_ctx->options = (_ctx->options & ~SSL_OP_ALL);

	if (![[OFUserDefaults sharedUserDefaults] boolForKey:HTTPSTLSv1Enabled])
		SSL_CTX_set_options(_ctx, SSL_OP_NO_TLSv1);
	else
		_ctx->options = (_ctx->options & ~SSL_OP_NO_TLSv1);

	if (![[OFUserDefaults sharedUserDefaults] boolForKey:HTTPSSSLv2Enabled])
		SSL_CTX_set_options(_ctx, SSL_OP_NO_SSLv2);
	else
		_ctx->options = (_ctx->options & ~SSL_OP_NO_SSLv2);

	if (![[OFUserDefaults sharedUserDefaults] boolForKey:HTTPSSSLv3Enabled])
		SSL_CTX_set_options(_ctx, SSL_OP_NO_SSLv3);
	else
		_ctx->options = (_ctx->options & ~SSL_OP_NO_SSLv3);
}

- (void)dealloc
{
	if (sharedCTXHandler == self)
		sharedCTXHandler = nil;

	if (_ctx != NULL)
	{
		SSL_CTX_free(_ctx);
		_ctx = NULL;
	}
	
	[super dealloc];
}

- (SSL_CTX *)ctx
{
	return _ctx;
}

@end
