//************************************************************************
//
//	HTTPSProcessor.m
//
//	by Jrgen Mllenhoff <jm@oic.de>		
//
//
//	This code is supplied "as is" the author makes no warranty as to its 
//	suitability for any purpose.  This code is free and may be distributed 
//	in accordance with the terms of the:
//		
//			GNU GENERAL PUBLIC LICENSE
//			Version 2, June 1991
//			copyright (C) 1989, 1991 Free Software Foundation, Inc.
// 			675 Mass Ave, Cambridge, MA 02139, USA
//
//************************************************************************

#import "HTTPSProcessor.h"

#import "HTTPSInformation.h"
#import "SSLSocket.h"
#import "HTTPSPreferences.h"
#import "HTTPSFunctions.h"
#import "OWTask+HTTPSExtension.h"
#import "HTTPSUtilities.h"

#import	<OmniBase/rcsid.h>
#import	<OmniBase/OBUtilities.h>

#import	<OmniFoundation/OmniFoundation.h>
#import <OWF/OWF.h>

#import <OmniNetworking/ONHost.h>
#import <OmniNetworking/ONSocketStream.h>


RCS_ID("$Header: /LocalDeveloper/CVS/HTTPS/HTTPSProcessor.m,v 1.13 1998/06/09 19:24:53 jurgen Exp $")

@interface OWHTTPProcessor(PrivateExtension)

- (void)connect;
- (void)disconnect;
- userAgentHeaderString;
- authorizationString;

@end

@implementation HTTPSProcessor

static	NSLock			*infoUserLock = nil;

static	NSString		*HTTPSTunnelingString = nil;
static	NSString		*HTTPSStatusString = nil;
static	NSString		*HTTPSModifiedStatusString = nil;

+ (void)initialize 
{
	static BOOL fInitialized = NO;

	[super initialize];
	if (fInitialized)
		return;
    fInitialized = YES;

	infoUserLock = [[NSLock alloc] init];

	HTTPSTunnelingString = [NSLocalizedStringFromTableInBundle(@"HTTPSTunnelingString", nil, CLASS_BUNDLE, @"Tunneling string.") retain];
	HTTPSStatusString = [NSLocalizedStringFromTableInBundle(@"HTTPSStatusString", nil, CLASS_BUNDLE, @"Status string.") retain];
	HTTPSModifiedStatusString = [NSLocalizedStringFromTableInBundle(@"HTTPSModifiedStatusString", nil, CLASS_BUNDLE, @"Modified Status string.") retain];
}

+ (int)defaultPort
{
	return 443;
}

- (void)_infoUser
{
	[[HTTPSInformation sharedHTTPSInformation] runModalWithHTTPSProcessor:self];

    [infoUserLock unlock]; // Unblocks the -connect method in the other thread
}

- (void)_logConnectionInfo
{
	SSLSocket	*sslSocket = (SSLSocket *)[socketStream socket];

	NSLog(@"SSL connection: URL=[%@], Cipher=[%@] Protocol=[%@] Keylen=[%d bits]", 
		[url netLocation], [sslSocket cipherString], [sslSocket protocolString], [sslSocket bits]);
}

- (void)_connectionInfos
{
	BOOL	fAbort = NO;
	static	BOOL disablePanel = NO;

	if ([[OFUserDefaults sharedUserDefaults] boolForKey:HTTPSDebugLog])
		[self _logConnectionInfo];

	if (![[self pipeline] isTopTask])
		return;

	if ([[OFUserDefaults sharedUserDefaults] boolForKey:HTTPSWarningPanelEnabled] && !disablePanel)
	{
		if (![[[self pipeline] target] isKindOfClass:NSClassFromString(@"OWBookmarkEntry")])
		{	// No Panel for Bookmark checking
			[infoUserLock lock];
    		[self mainThreadPerformSelector:@selector(_infoUser)];
    		[infoUserLock lock]; // This will block until _infoUser unlocks it from the main thread
			fAbort = [[HTTPSInformation sharedHTTPSInformation] canceled];
			disablePanel = [[HTTPSInformation sharedHTTPSInformation] disablePanel];
    		[infoUserLock unlock];

			if (fAbort)
			{
				// [OWPipeline abortPipelinesForTarget:[[self pipeline] target]];
				[NSException raise:HTTPSProcessorUserAbortExceptionName format:@"HTTPSProcessor: Request canceled by user"];
			}
		}
	}
	else
	{
		if (![[OFUserDefaults sharedUserDefaults] boolForKey:HTTPSDebugLog])
			[self _logConnectionInfo];
	}
}

- (BOOL)_tunnelConnectToHostname:(NSString *)hostname port:(int)port
{
	BOOL			fConnect = NO;
	OWNetLocation	*proxyNetLocation;
	ONHost			*proxyHost;
	NSString		*portString, *returnString; 
	NSMutableString	*connectString;
	int				proxyPort;
	
	[self setStatusFormat:HTTPSTunnelingString, [url netLocation]];

	// Get the OWNetLocation for the ProxyURL
	proxyNetLocation = [proxyURL parsedNetLocation];

	// Create the Proxy-ONHost for the Proxy-OWNetLocation
	proxyHost = [ONHost hostForHostname:[proxyNetLocation hostname]];
	
	// Which port for the proxy?
	portString = [proxyNetLocation port];
	proxyPort = (portString != nil) ? [portString intValue] : [[self class] defaultPort];

	// Connect to the proxy
	[(SSLSocket *)[socketStream socket] connectToHost:proxyHost port:proxyPort];

	// Build CONNECT-String	
	connectString = [NSMutableString stringWithFormat:@"CONNECT %@:%d HTTP/1.0\r\n%@", 
		hostname, port, [self userAgentHeaderString]];

	// Need Authorization?
	if ([self authorizationString] != nil)
		[connectString appendString:[self authorizationString]];

	// And an empty line for the end
	[connectString appendString:@"\r\n"];

	// Start tunneling
	[socketStream writeString:connectString];

	if ([[OFUserDefaults sharedUserDefaults] boolForKey:HTTPSDebugLog])
		NSLog(@"SSLTunneling->ConnectString: [%@]", connectString);

	while(YES)
	{
		returnString = [socketStream peekLine];

		if ([[OFUserDefaults sharedUserDefaults] boolForKey:HTTPSDebugLog])
			NSLog(@"SSLTunneling->ReturnString: [%@]", returnString);

		if ([returnString length] <= 0 || fConnect)
		{
			returnString = [socketStream readLine];
			if ([returnString length] <= 0)
				break; // Empty line

			continue; // Read the stuff behind the "200 CONNECTION" Message 
		}

		if (![returnString hasPrefix:@"HTTP/"])
		{
			returnString = [socketStream readLine];
			continue;
		}
	
		// Connect successful
		if ([returnString containsString:@" 200 "])
		{
			fConnect = YES;
			returnString = [socketStream readLine];
			// Read until the empty line
			continue;
		}
		
		// Other response then 200, maybe an error or proxy authorization?
		break;	// OWHTTPProcessor should handle this
	}
	
	return fConnect;
}

- (void)connect
{
	BOOL			fStartConnection = NO;
	OWNetLocation	*netLocation;
	NSString		*portString;
	SSLSocket		*sslSocket;
	int				port;
	NSException		*raisedException = nil;

	[self setStatusFormat:HTTPSStatusString, [url netLocation]];

	if (![[OFUserDefaults sharedUserDefaults] boolForKey:HTTPSCacheEnabled])
		[[self pipeline] disableContentCache];

	// Get SSLSocket
	sslSocket = [SSLSocket sslSocket];
	[sslSocket setReadBufferSize:32*1024];

	// Create SocketStream
	socketStream = [[ONSocketStream alloc] initWithSocket:sslSocket];

	// Get the OWNetLocation for the URL
	netLocation = [url parsedNetLocation];

	// Which Port?
	portString = [netLocation port];
	port = (portString != nil) ? [portString intValue] : [HTTPSProcessor defaultPort];

	NS_DURING
		if (url == proxyURL)
		{
			if ([[OFUserDefaults sharedUserDefaults] boolForKey:HTTPSDebugLog])
				NSLog(@"HTTPSProcessor->URL: [%@]", url);

			// Direct
			[sslSocket connectToHost:[ONHost hostForHostname:[netLocation hostname]] port:port];
			fStartConnection = YES;
		}
		else
		{
			if ([[OFUserDefaults sharedUserDefaults] boolForKey:HTTPSDebugLog])
				NSLog(@"HTTPSProcessor->URL: [%@], ProxyURL: [%@]", url, proxyURL);

			// Proxy
			if ([self _tunnelConnectToHostname:[netLocation hostname] port:port])
			{
				// Hack! Without these lines a URL like https://vault.sfnb.com/cgi-bin/accntSum
				// doesn't work. I don't know why....
				[proxyURL release];
				proxyURL = [url retain];
				fStartConnection = YES;
			}
		}

		if (fStartConnection)
		{
			// Open the SSL-Connection
			[sslSocket sslConnect];
			[self _connectionInfos];
		}
	NS_HANDLER
		[self setStatusString:[localException reason]];
		raisedException = localException;
	NS_ENDHANDLER

	if (raisedException != nil)
		[raisedException raise];
}

- (void)setStatusString:(NSString *)newStatusString
{
SSLSocket	*sslSocket = (SSLSocket *)[socketStream socket];

	if ([sslSocket isSecureConnection])
	{
		newStatusString = [NSString stringWithFormat:HTTPSModifiedStatusString, 
			newStatusString, [sslSocket protocolString], [sslSocket bits]];
	}

	[super setStatusString:newStatusString];
}

- (ONSocketStream *)socketStream
{
	return socketStream;
}

- (OWAddress *)address
{
	return address;
}

@end

DEFINE_NSSTRING(HTTPSProcessorUserAbortExceptionName);
